<!-- 
This README describes the package. If you publish this package to pub.dev,
this README's contents appear on the landing page for your package.

For information about how to write a good package README, see the guide for
[writing package pages](https://dart.dev/guides/libraries/writing-package-pages). 

For general information about developing packages, see the Dart guide for
[creating packages](https://dart.dev/guides/libraries/create-library-packages)
and the Flutter guide for
[developing packages and plugins](https://flutter.dev/developing-packages). 
-->

Provides a Resource class containing data of any type and an error of any type. This allows methods
to return a `Resource<T>` which can be a `Success`, `Loading` or `Failure` instead of returning
a `T` and throwing an `Exception` if the method fails. Additionally, if a method returns a `Future`
or `Stream` the `Loading` state can initially be returned.

<!--
## Features

TODO: List what your package can do. Maybe include images, gifs, or videos.

## Getting started

TODO: List prerequisites and provide or point to information on how to start using the package.
-->

## Usage

### To return a resource from a method:

Synchronous

```dart
Resource<int> parseInt(String value) {
  try {
    return Success(int.parse(value));
  } catch (e) {
    return Failure.fromMessage("Failed to parse value '$value'");
  }
}
```

Asynchronous

```dart
convert(someStream).listen((resource) {
    resource.resolve(
        onSuccess: (someClassData) => Text(
            "Converted someClass to otherClass instance successfully: $someClassData"),
        onLoading: (_) => Text("Loading..."),
        onFailure: (error) => Text("Failed to convert: $error")
    );
});
```

### Use Resource result

```dart
final Resource<int> result = parseInt("13");
if (result.hasData) {
  Text("Parsed int successfully: ${result.data}");
} else {
  Text("Failed to parse int: ${result.error?.message}");
}
//or
Stream<Resource<OtherClass>> convert(Stream<SomeClass> toConvert) async* {
  yield Loading(EmptyOtherClass());
  await for (final someObject in toConvert) {
    try {
      final OtherClass converted = convertToSomeObject(someObject);
      yield Success(converted);
    } catch (e) {
      yield Failure.fromMessage("Failed convert '$someObject'");
    }
  }
}
``` 

<!--
## Additional information

TODO: Tell users more about the package: where to find more information, how to contribute to the
package, how to file issues, what response they can expect from the package authors, and more.
-->