class Resource<T> {
  T? data;
  Error? error;

  Resource({this.data, this.error});

  bool get hasData => data != null;

  void resolve(
      {required dynamic Function(T data) onSuccess,
      dynamic Function(T? data)? onLoading,
      dynamic Function(Error error)? onFailure}) {
    if (this is Success) {
      onSuccess(data!);
    } else if (onLoading != null && this is Loading) {
      onLoading(data);
    } else if (onFailure != null && this is Failure) {
      onFailure(error!);
    }
  }
}

class Success<T> extends Resource<T> {
  Success(T data) : super(data: data);
}

class Loading<T> extends Resource<T> {
  Loading(T? data) : super(data: data);
}

class Failure<T> extends Resource<T> {
  Failure(
    Error error, {
    T? data,
  }) : super(data: data, error: error);

  Failure.fromMessage(
      String errorMessage, {
        T? data,
      }) : super(data: data, error: Error(errorMessage));
}

class Error{
  final String? message;

  Error(this.message);
}
